/**
 * @file	cf_socket.h
 * @author	myusgun <myusgun@gmail.com>
 * @version	0.1
 *
 * @remark TCP 소켓만 지원됨
 *
 * @example socket_server.c
 * @example socket_client.c
 */
#ifndef __CF_SOCKET_H__
#define __CF_SOCKET_H__

#include "cf_base.h"
#include <stddef.h>

#ifdef _WIN32
# include <WinSock2.h>
#else
# include <netinet/in.h>
# include <sys/socket.h>
# include <arpa/inet.h>
# include <netdb.h>
# include <unistd.h>
#endif

#ifdef _WIN32
typedef int socklen_t;
#endif // #ifdef _WIN32

#ifdef __cplusplus
extern "C" {
#endif

CF_EXPORT CF_BOOL
CF_Socket_IsInitialized	(void);

CF_EXPORT int
CF_Socket_Initialize	(void);

CF_EXPORT int
CF_Socket_Finalize		(void);

CF_EXPORT int
CF_Socket_Close			(const int sock);

CF_EXPORT int
CF_Socket_SetOption		(const int		sock,
						 const int		optname,
						 const void		* optval,
						 const size_t	optlen);

CF_EXPORT int
CF_Socket_GetOption		(const int	sock,
						 const int	optname,
						 void		* optval,
						 size_t		* optlen);

CF_EXPORT int
CF_Socket_SetTimeout	(const int	sock,
						 const int	timeout);

CF_EXPORT int
CF_Socket_Connect		(const char				* ip,
						 const unsigned short	port);

CF_EXPORT int
CF_Socket_Server		(const unsigned short	port,
						 const int				backlog);

CF_EXPORT int
CF_Socket_Accept		(const int			sock,
						 struct sockaddr_in	* address);

CF_EXPORT int
CF_Socket_Send			(const int	sock,
						 const void	* buf,
						 const int	len);

CF_EXPORT int
CF_Socket_Recv			(const int	sock,
						 void		* buf,
						 const int	len);

#ifdef __cplusplus
}
#endif

#endif // #ifndef __CF_SOCKET_H__
